#!/usr/bin/env python3
import gi, threading, subprocess
gi.require_version("Gtk", "4.0")
gi.require_version("Adw", "1")
from gi.repository import Gtk, Adw, GLib

class MyApp(Adw.Application):
    def do_activate(self):
        builder = Gtk.Builder.new_from_file("app.ui")
        window = builder.get_object("window")
        self.window = window
        window.set_application(self)

        # Connect start and cancel button signals
        self.start_button = builder.get_object("button_show_banner")
        self.cancel_button = builder.get_object("cancel_button")
        self.button_container = self.start_button.get_parent()
        self.start_button.connect("clicked", self.on_start_clicked)
        self.cancel_button.connect("clicked", self.on_cancel_clicked)

        window.present()

    def on_start_clicked(self, button):
        # Disable buttons during upgrade
        self.start_button.set_sensitive(False)
        self.cancel_button.set_sensitive(False)
        self.progress_bar = Gtk.ProgressBar()
        self.progress_bar.set_show_text(False)
        self.button_container.append(self.progress_bar)
        self.upgrade_running = True
        GLib.timeout_add(100, self.on_pulse_timeout)
        threading.Thread(target=self.run_upgrade, daemon=True).start()

    def on_pulse_timeout(self):
        if self.upgrade_running:
            self.progress_bar.pulse()
            return True
        else:
            self.progress_bar.set_fraction(1.0)
            return False

    def run_upgrade(self):
        cmd = (
            "pkexec dnf remove -y alsa-sof-firmware wine* gstreamer1-plugins-bad-freeworld "
            "gstreamer1-plugins-ugly anaconda-install-img-deps virt-manager && "
            "pkexec dnf distro-sync -y --allowerasing --best --releasever=10 --disablerepo=* "
            "--repofrompath=oreon,https://download.copr.fedorainfracloud.org/results/brandonlester/oreon-10/centos-stream-10-x86_64/ "
            "--repofrompath=oreonmirror,https://packages.boostyconnect.com/oreon-10/x86_64/ "
            "--repofrompath=oreonextras,https://packages.boostyconnect.com/oreon-10/extras-x86_64/ "
            "--repofrompath=fedoraepel,https://dl.fedoraproject.org/pub/epel/10/Everything/x86_64/ "
            "--repofrompath=base,https://kitten.repo.almalinux.org/10-kitten/BaseOS/x86_64/os/ "
            "--repofrompath=Oreonappstream,https://kitten.repo.almalinux.org/10-kitten/AppStream/x86_64/os/ "
            "--repofrompath=Oreonextras,https://kitten.repo.almalinux.org/10-kitten/extras-common/x86_64/os/ "
            "--nogpgcheck"
        )
        subprocess.run(cmd, shell=True)
        GLib.idle_add(self.on_upgrade_finished)

    def on_upgrade_finished(self):
        self.upgrade_running = False
        # Remove progress bar using unparent() since destroy() is not available in GTK4
        self.progress_bar.unparent()
        # Re-enable cancel button after upgrade completes
        self.cancel_button.set_sensitive(True)
        self.reboot_button = Gtk.Button.new_with_label("Reboot")
        self.reboot_button.connect("clicked", self.on_reboot_clicked)
        self.button_container.append(self.reboot_button)
        return False

    def on_reboot_clicked(self, button):
        subprocess.run("reboot", shell=True)

    def on_cancel_clicked(self, button):
        self.window.close()

    def do_startup(self):
        Adw.Application.do_startup(self)

app = MyApp()
app.run(None)

